@@ -1,7 +1,7 @@
 // -*- c++ -*-
 #ifndef _GLIBMM_HELPERLIST_H
 #define _GLIBMM_HELPERLIST_H
-/* $Id: helperlist.h 317 2006-08-18 14:59:17Z  $ */
+/* $Id: helperlist.h 387 2007-03-23 17:31:51Z murrayc $ */
 
 /* helperlist.h
  *
@@ -36,8 +36,8 @@ public:
   : gparent_(0)
   {}
 
-  HelperList(GObject* gparent)
-  : gparent_(gparent)
+  HelperList(GObject* gp) //We use gp instead of gparent because that can cause warnings about a shadowed member.
+  : gparent_(gp)
   {}
 
   virtual ~HelperList()
